;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; App var routines                                              ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; loadAppvar - find & load the appvar, or create it if it doesn't exist, or update it if a previous version exists
;   input:  none
;   output: Z = loaded successfully
;------------------------------------------------
loadAppvar:
        ld      hl,dataAppvar
        MOVE9()                                 ; (OP1) = name of appvar to be checked for and created if necessary
        bcall(_chkfindsym)
        ld      ix,miscFlags
        jr      nc,dataAppvarFound              ; if it's already on the calc, go to validate it
        ; otherwise, let's create it and put the default high score, etc. data in it
        set     appvarFlag,(ix)
        call    createAppvar
        jr      dataAppvarRam
dataAppvarFound:
        res     appvarFlag,(ix)
#ifdef TI83PLUS
        ld      a,b                             ; A = rom page (or 0 if in ram)
        or      a                               ; check if in ram
#endif
#ifdef TI84PLUSCE
        call    _chkInRam
#endif
        jr      z,dataAppvarRam                 ; if so, no need to unarchive
        bcall(_arc_unarc)                       ; unarchive appvar (name is still in OP1)
        jr      loadAppvar                      ; go find the appvar again (which will now be in ram)
dataAppvarRam:
        inc     de
        inc     de                              ; DE => data version
        ld      a,(de)
        cp      DATA_APPVAR_VERSION             ; check that it's the current version
        jr      nz,updateAppvar                 ; if it wasn't, try to update it
        inc     de                              ; DE => high score data
        ld      (pHscores),de                   ; save ptr
        ld      hl,HSCORE_DATA_SIZE
        add     hl,de                           ; HL => options flags
        ld      (pOptions),hl                   ; save ptr
        inc     hl                              ; HL => tips data
        ld      (pTips),hl                      ; save ptr
        inc     hl                              ; HL => achievement flags
        ld      (pAchFlags),hl                  ; save ptr
        ld      de,ACH_FLAGS_SIZE
        add     hl,de                           ; HL => achievement variables
        ld      (pAchVars),hl
        ret                                     ; Z flag will be set from the earlier CP, confirming that the appvar has been successfully loaded

;------------------------------------------------
; updateAppvar - attempt to update an old version appvar to current version
;   input:  DE => data version
;           A = data version
;           HL => vat entry
;   output: NZ if unable to update, otherwise will jump back to loadAppvar to load the updated version, thus returning Z
;------------------------------------------------
updateAppvar:
        cp      5                               ; 2.1.0 appvar?
        jr      z,updateAppvar5
        cp      4                               ; 2.0.0 appvar?
        jr      z,updateAppvar4
        cp      3                               ; this is the value that will be in all pre v2 appvars
        jp      nz,appvarError                  ; if it's not pre v2, then it must be a future version, and we can't handle that!
updateAppvar3:
        ; update from version 3 to 4 (pre v2 -> 2.0.0)
        ld      bc,55
        push    de
        pop     ix
        dec     de
        ld      a,4
        call    recreateAppvar                  ; recreate the appvar at the larger size using existing 55 bytes of data
        ld      a,4
        dec     de
        ld      (de),a                          ; set data version to 4 (next time round it will run the 4->5 update)
        inc     de
        ; next, the cursor coords in version 3 were different, so the version 4 ones need to be put back in again
        inc     de                              ; DE => 1st high score cursor coords in appvar
        ld      hl,highScoreNames+1             ; HL => 1st high score cursor coords that we want to use
        ld      b,3                             ; B = 3 sets of coords to copy
updateAppvar3Cursor:
        push    bc
        push    de
        push    hl
        ld      bc,2                            ; BC = 2 bytes to copy
        ldir                                    ; copy the cursor coords
        pop     hl
        pop     de
        ld      bc,16
        add     hl,bc                           ; HL => next source
        ex      de,hl
        add     hl,bc
        ex      de,hl                           ; DE => next destination
        pop     bc
        djnz    updateAppvar3Cursor
        jp      loadAppvar                      ; go back and load the updated appvar
updateAppvar4:
        ; update from version 4 to 5 (2.0.0 -> 2.1.0)
        ld      bc,60
        push    de
        pop     ix
        ld      a,5
        call    recreateAppvar
        jp      loadAppvar
updateAppvar5:
        ; update from version 5 to 6 (2.1.0 -> 3.0.0)
        ld      bc,92
        push    de
        pop     ix
        ld      a,6
        call    recreateAppvar
        ; push some of the old achievement data down
        ex      de,hl                           ; HL => start of high scores, ie. start of appvar data after the version byte
        push    hl
        ld      de,appvarBuffer
        ld      bc,DATA_APPVAR_SIZE-1
        ldir                                    ; load appvar data to appvarBuffer for data moving
        ; push items pg1 counters
        ld      hl,appvarBuffer+77
        ld      de,appvarBuffer+96
        ld      bc,15
        call    moveBytes
        ; push kills pg1 counters
        ld      hl,appvarBuffer+62
        ld      de,appvarBuffer+66
        ld      bc,15
        call    moveBytes
        ; push items pg1 flags
        ld      hl,appvarBuffer+61
        ld      de,appvarBuffer+64
        ld      bc,1
        call    moveBytes
        ; push kills pg1 flags
        ld      hl,appvarBuffer+60
        ld      de,appvarBuffer+62
        ld      bc,1
        call    moveBytes
        ; push ep1 & ep2 flags
        ld      hl,appvarBuffer+58
        ld      de,appvarBuffer+59
        ld      bc,2
        call    moveBytes
        ; load existing progress into new achievements
        ; enemy kills
        ld      hl,appvarBuffer+75
        ld      de,appvarBuffer+81
        ld      bc,3
        ldir
        ; boss kills
        ld      hl,appvarBuffer+78
        push    hl
        ld      de,appvarBuffer+84
        ld      bc,3
        push    bc
        ldir
        pop     bc
        pop     hl
        ldir
        ; credits
        ld      hl,appvarBuffer+102
        ld      de,appvarBuffer+111
        ld      bc,3
        ldir
        ; key passes
        ld      hl,appvarBuffer+105
        ld      de,appvarBuffer+120
        ld      bc,3
        ldir
        ; ammo clips
        ld      hl,appvarBuffer+108
        ld      de,appvarBuffer+123
        ld      bc,3
        ldir
        ; load updated appvar data back to appvar file
        pop     de
        ld      hl,appvarBuffer
        ld      bc,DATA_APPVAR_SIZE-1
        ldir
        jp      loadAppvar

;------------------------------------------------
; appvarError - display an error message about the appvar
;   input:  none
;   output: NZ to signify the error back to the main program routine
;------------------------------------------------
appvarError:
#ifdef TI83PLUS
        bcall(_grbufclr)
        ld      hl,strAppvarError
        call    showStrings
        bcall(_grbufcpy)
#endif
        call    waitKey
        inc     a                               ; set NZ (guaranteed as A will currently equal the GET_KEY value)
        ret

;------------------------------------------------
; recreateAppvar - delete and recreate the appvar at the current size with existing data
;   input:  DE => data version
;           HL => vat entry
;           IX => start of appvar data (little workaround for pre v2 appvar)
;           BC = # bytes to save from existing appvar (excluding version byte)
;           A = new data version
;   output: DE => start of high scores
;------------------------------------------------
recreateAppvar:
        push    af
        push    hl
        push    ix
        inc     de
        ex      de,hl                           ; HL => start of high score data
        ld      de,appvarBuffer                 ; DE => buffer to copy information to
        push    bc                              ; save byte counter
        ldir                                    ; save the old format data to the buffer
        pop     bc                              ; restore byte counter
        pop     de
        dec     de
        dec     de                              ; DE => start of appvar data storage (including size bytes)
        pop     hl                              ; HL => vat entry
        push    bc                              ; save byte counter
        ld      b,0                             ; B = 0 as appvar is located in ram
        bcall(_delvar)                          ; delete old version appvar
        call    createAppvar                    ; create new (default) appvar, to which we can then load in the old relevant data
        pop     bc                              ; restore byte counter
        inc     de
        inc     de
        pop     af
        ld      (de),a                          ; set data version that we've updated to
        inc     de                              ; DE => where high scores should be
        push    de                              ; save this for after
        ld      hl,appvarBuffer                 ; HL => buffered data (the high scores from the old version appvar)
        ldir                                    ; load the old high scores into the new format appvar
        pop     de                              ; DE => where high scores should be
        ret

;------------------------------------------------
; moveBytes - move a chunk of bytes and fill the old location with zeros
;   input:  HL => data to move
;           DE => destination
;           BC => # of bytes to move (must be <= 256)
;------------------------------------------------
moveBytes:
        push    de
        push    hl
        push    bc
        ld      de,appvarBuffer+512
        ldir                                    ; copy data to buffer
        pop     bc
        pop     hl
        push    bc
        dec     bc
        push hl \ pop de
        inc     de
        ld      (hl),0
        ld      a,b
        or      c
        jr      z,moveBytesSkipFill
        ldir                                    ; zero-fill old location
moveBytesSkipFill:
        pop     bc
        ld      hl,appvarBuffer+512
        pop     de
        ldir                                    ; copy data from buffer to new location
        ret

;------------------------------------------------
; createAppvar - create a new appvar with default high scores, etc.
;   input:  (OP1) = appvar name
;   output: DE => start of appvar in RAM (size bytes)
;------------------------------------------------
createAppvar:
        ld      hl,DATA_APPVAR_SIZE             ; HL = size of appvar to be created
        bcall(_createappvar)
        push    de                              ; save ptr to appvar data (size bytes)
        inc     de
        inc     de                              ; DE => start of appvar actual data storage
        ld      hl,appvarDefault                ; HL => default high score data
        ld      bc,DATA_APPVAR_SIZE             ; BC = bytes to copy
        ldir                                    ; load default high score data into appvar
        pop     de                              ; DE => restore ptr
        ret

.end
