;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Graphics routines (83+ only)                                  ;
;                                                               ;
;===============================================================;

#define BOTTOM_CLIP 64
#define RIGHT_CLIP 96

;------------------------------------------------
; putAlignedSprite - Draw an 8x8 aligned sprite
;   input:  DE => sprite
;           A = x coord
;           L = y coord
;   output: DE => next sprite
;------------------------------------------------
putAlignedSprite:
        ld      h,0
        add     hl,hl                           ; x2
        add     hl,hl                           ; x4
        add     hl,hl                           ; x8
        add     hl,hl                           ; x16
        add     hl,hl                           ; x32
        ld      b,h
        ld      c,l
        add     hl,hl                           ; x64
        add     hl,bc                           ; x96
        ld      c,a
        ld      b,0
        add     hl,bc
        ld      bc,gbuf
        add     hl,bc                           ; HL => Where to put sprite
        ld      b,8
putASLoop:
        ld      a,(de)
        ld      (hl),a
        inc     de
        push    de
        ld      de,12
        add     hl,de
        pop     de
        djnz    putASLoop
        ret

;------------------------------------------------
; putSprite - Draw an 8x8 sprite to gbuf (using XOR method), no clipping
;   input:  DE => sprite
;           A = x coord
;           L = y coord
;   output: IX => next sprite
;------------------------------------------------
putSprite:
        ld      h,0
        ld      b,h
        add     hl,hl                           ; x2
        add     hl,hl                           ; x4
        ld      c,l
        add     hl,hl                           ; x8
        add     hl,bc                           ; x12
        ld      c,a
        srl     c
        srl     c
        srl     c
        add     hl,bc
        ld      bc,gbuf
        add     hl,bc                           ; HL => Where to display sprite
        and     $07
        ld      c,a
        push    de
        pop     ix                              ; IX => Sprite
        ld      b,8
putSLoop:
        ld      d,(ix)
        ld      e,0
        ld      a,c
        or      a
        jr      z,putSSkip
putSLoop2:
        srl     d
        rr      e
        dec     a
        jr      nz,putSLoop2
putSSkip:
        ld      a,(hl)
        xor     d
        ld      (hl),a
        inc     hl
        ld      a,(hl)
        xor     e
        ld      (hl),a
        ld      de,11
        add     hl,de
        inc     ix
        djnz    putSLoop
        ret

;------------------------------------------------
; putClippedSprite - Draw an 8x8 unmasked sprite to gbuf with clipping
;   input:  HL => sprite
;           B = x coord
;           C = y coord
;   output: none
;   notes:  thanks to Jimmy Mardell
;------------------------------------------------
putClippedSprite:
        ld      a,c
        cp      200
        jr      nc,putCSNoBottomClip
        cp      BOTTOM_CLIP
        ret     nc
putCSNoBottomClip:
        ld      a,$FF
        ld      (putCSClipMask),a
        bit     7,b
        jr      z,putCSCheckRightClip
        ld      a,b
        cp      249
        ret     c
        neg
        push    bc
        ld      b,a
        ld      a,$FF
putCSLeftClip:
        srl     a
        djnz    putCSLeftClip
        ld      (putCSClipMask),a
        pop     bc
        ld      a,b
        add     a,96
        ld      b,a
        dec     c
        jr      putCSCheckBottomClip
putCSCheckRightClip:
        ld      a,b
        sub     RIGHT_CLIP-7
        jr      c,putCSCheckBottomClip
        push    bc
        ld      b,a
        inc     b
        ld      a,$FF
putCSRightClip:
        add     a,a
        djnz    putCSRightClip
        ld      (putCSClipMask),a
        pop     bc
putCSCheckBottomClip:
        ld      a,8
        ld      (putCSRows2Put),a
        bit     7,c
        jr      nz,putCSCheckTopClip
        ld      a,BOTTOM_CLIP-1
        sub     c
        ret     c
        inc     a
        cp      8
        jr      nc,putCSClippingDone
        ld      (putCSRows2Put),a
        jr      putCSClippingDone
putCSCheckTopClip:
        ld      a,c
        cp      249
        ret     c
        push    bc
        neg
        ld      b,a
        sub     8
        neg
        ld      (putCSRows2Put),a
putCSTopClip:
        inc     hl
        djnz    putCSTopClip
        pop     bc
        ld      c,0
putCSClippingDone:
        push    hl
        pop     ix                              ; IX => Sprite
        ld      a,(putCSRows2Put)
        push    af
        call    findPixel
        ld      (putCSBitMask),a
        pop     bc
putCSPutRow:
        push    bc
        push    hl
        ld      a,(putCSClipMask)
        ld      e,a
        ld      b,8
        ld      c,(ix)
        inc     ix
putCSPutCol:
        push    bc
        ld      a,(putCSBitMask)
        and     (hl)
        rlc     e
        jr      nc,putCSNextBit
        ld      a,(putCSBitMask)
        rlc     c
        jr      c,putCSBitOn
        jr      putCSNextBit
putCSBitOn:
        or      (hl)
        ld      (hl),a
putCSNextBit:
        ld      a,(putCSBitMask)
        rrca
        ld      (putCSBitMask),a
        jr      nc,putCSNoIncHL
        inc     hl
putCSNoIncHL:
        pop     bc
        rlc     c
        djnz    putCSPutCol
        pop     hl
        ld      de,12
        add     hl,de
        pop     bc
        djnz    putCSPutRow
        ret

;------------------------------------------------
; putClippedMaskedSprite - Draw an 8x8 masked sprite to gbuf with clipping
;   input:  HL => Sprite followed by mask
;           B = x
;           C = y
;   output: none
;   notes:  thanks to Jimmy Mardell
;------------------------------------------------
putClippedMaskedSprite:
        ld      a,c
        cp      200
        jr      nc,putCMSNoBottomClip
        cp      BOTTOM_CLIP
        ret     nc
putCMSNoBottomClip:
        ld      a,$FF
        ld      (putCMSClipMask),a
        bit     7,b
        jr      z,putCMSCheckRightClip
        ld      a,b
        cp      249
        ret     c
        neg
        push    bc
        ld      b,a
        ld      a,$FF
putCMSLeftClip:
        srl     a
        djnz    putCMSLeftClip
        ld      (putCMSClipMask),a
        pop     bc
        ld      a,b
        add     a,96
        ld      b,a
        dec     c
        jr      putCMSCheckBottomClip
putCMSCheckRightClip:
        ld      a,b
        sub     RIGHT_CLIP-7
        jr      c,putCMSCheckBottomClip
        push    bc
        ld      b,a
        inc     b
        ld      a,$FF
putCMSRightClip:
        add     a,a
        djnz    putCMSRightClip
        ld      (putCMSClipMask),a
        pop     bc
putCMSCheckBottomClip:
        ld      a,8
        ld      (putCMSRows2Put),a
        bit     7,c
        jr      nz,putCMSCheckTopClip
        ld      a,BOTTOM_CLIP-1
        sub     c
        ret     c
        inc     a
        cp      8
        jr      nc,putCMSClippingDone
        ld      (putCMSRows2Put),a
        jr      putCMSClippingDone
putCMSCheckTopClip:
        ld      a,c
        cp      249
        ret     c
        push    bc
        neg
        ld      b,a
        sub     8
        neg
        ld      (putCMSRows2Put),a
putCMSTopClip:
        inc     hl
        djnz    putCMSTopClip
        pop     bc
        ld      c,0
putCMSClippingDone:
        push    hl
        pop     ix                              ; IX => Sprite, Mask
        ld      a,(putCMSRows2Put)
        push    af
        call    findPixel
        ld      (putCMSBitMask),a
        pop     bc
putCMSPutRow:
        push    bc
        push    hl
        ld      a,(putCMSClipMask)
        ld      e,a
        ld      a,(ix+8)
        and     e
        ld      e,a
        ld      b,8
        ld      c,(ix)
        inc     ix
putCMSPutCol:
        push    bc
        sla     d
        ld      a,(putCMSBitMask)
        and     (hl)
        rlc     e
        jr      nc,putCMSNextBit
        ld      a,(putCMSBitMask)
        rlc     c
        jr      c,putCMSBitOn
        cpl
        and     (hl)
        ld      (hl),a
        jr      putCMSNextBit
putCMSBitOn:
        or      (hl)
        ld      (hl),a
putCMSNextBit:
        ld      a,(putCMSBitMask)
        rrca
        ld      (putCMSBitMask),a
        jr      nc,putCMSNoIncHL
        inc     hl
putCMSNoIncHL:
        pop     bc
        rlc     c
        djnz    putCMSPutCol
        pop     hl
        ld      de,12
        add     hl,de
        pop     bc
        djnz    putCMSPutRow
        ret

;------------------------------------------------
; findPixel - Get byte and bit offset to a pixel on gbuf
;   input:  B = X
;           C = Y
;   output: HL => Byte in gbuf
;           A = Bitmask
;------------------------------------------------
findPixel:
        ld      a,b
        ld      b,0
        ld      h,b
        ld      l,c                             ; HL = Y x 1
        add     hl,bc                           ; x2
        add     hl,bc                           ; x3
        add     hl,hl                           ; x6
        add     hl,hl                           ; x12
        ld      bc,gbuf
        add     hl,bc
        ld      b,0
        ld      c,a
        and     $07
        srl     c
        srl     c
        srl     c
        add     hl,bc
        ld      b,a
        inc     b
        ld      a,$01
findPixelLoop:
        rrca
        djnz    findPixelLoop
        ret

;------------------------------------------------
; gbufInvert - invert gbuf contents
;   input:  none
;   output: none
;------------------------------------------------
gbufInvert:
        ld      hl,gbuf
        ld      bc,768
gbufInvertLoop:
        ld      a,(hl)
        cpl
        ld      (hl),a
        inc     hl
        dec     bc
        ld      a,b
        or      c
        jr      nz,gbufInvertLoop
        ret

;------------------------------------------------
; gbufCopy - copy gbuf to lcd fast
;   input:  none
;   output: none
;   notes:  taken from Ion code, credit goes to JoeW
;------------------------------------------------
gbufCopy:
        di
        ld      a,$80                           ; 7
        call    lcdBusy
        out     ($10),a                         ; 11
        ld      hl,gbuf-12-(-(12*64)+1)         ; 10
        ld      a,$20                           ; 7
        ld      c,a                             ; 4
        inc     hl                              ; 6 waste
        dec     hl                              ; 6 waste
fastCopyAgain:
        ld      b,64                            ; 7
        inc     c                               ; 4
        ld      de,-(12*64)+1                   ; 10
        call    lcdBusy
        out     ($10),a                         ; 11
        add     hl,de                           ; 11
        ld      de,10                           ; 10
fastCopyLoop:
        add     hl,de                           ; 11
        inc     hl                              ; 6 waste
        inc     hl                              ; 6 waste
        inc     de                              ; 6
        ld      a,(hl)                          ; 7
        call    lcdBusy
        out     ($11),a                         ; 11
        dec     de                              ; 6
        djnz    fastCopyLoop                    ; 13/8
        ld      a,c                             ; 4
        cp      $2B+1                           ; 7
        jr      nz,fastCopyAgain                ; 10/1
        ret                                     ; 10

;------------------------------------------------
; lcdBusy - wait for lcd to be ready to receive commands
;   input:  none
;   output: none
;   notes:  preserves all registers
;------------------------------------------------
lcdBusy:
        push    bc
        ld      b,a
lcdBusyLoop:
        in      a,($10)
        rla
        jr      c,lcdBusyLoop
        ld      a,b
        pop     bc
        ret

.end
