;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Miscellaneous routines                                        ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; waitKey - wait for a key press
;   input:  none
;   output: A = key pressed (_getcsc style)
;------------------------------------------------
waitKey:
        ei                                      ; enable interrupts
        res     apdrunning,(iy+apdflags)        ; disable apd
wKeyLoop:
        halt
        bcall(GET_KEY)                          ; A = key pressed
        or      a                               ; check if 0
        jr      z,wKeyLoop                      ; if so, no key was pressed
        set     apdrunning,(iy+apdflags)        ; enable apd
        ret

;------------------------------------------------
; wait2ndEnter - wait for [2nd] or [ENTER]
;   input:  none
;   output: none
;------------------------------------------------
wait2ndEnter:
        call    waitKey
        cp      GK_ENTER
        ret     z
        cp      GK_2ND
        ret     z
        jr      wait2ndEnter

#ifdef TI84PLUSCE
;------------------------------------------------
; wait2ndEnterSto - wait for [2nd], [ENTER] or [STO]
;   input:  none
;   output: A = keycode
;------------------------------------------------
wait2ndEnterSto:
        call    waitKey
        cp      GK_ENTER
        ret     z
        cp      GK_2ND
        ret     z
        cp      GK_STO
        ret     z
        jr      wait2ndEnterSto
#endif

;------------------------------------------------
; delay - delay for a specified time
;   input:  B = number of times to halt
;   output: none
;------------------------------------------------
delay:
        res     apdrunning,(iy+apdflags)        ; disable apd
delayLoop:
        ei
        halt
        djnz    delayLoop
        ret

;------------------------------------------------
; shortDelay - delay for a specified time (can be skipped by user)
;   input:  B = delay length
;   output: CA = [ALPHA] was pressed, otherwise wasn't pressed
;------------------------------------------------
shortDelay:
        ld      c,b
        ld      b,0
        ; fall through to longDelay

;------------------------------------------------
; longDelay - delay for a specified time (can be skipped by user)
;   input:  BC = delay length
;   output: CA = [ALPHA] was pressed, otherwise wasn't pressed
;------------------------------------------------
longDelay:
        res     apdrunning,(iy+apdflags)        ; disable apd
longDelayLoop:
        push    bc                              ; save counter
#ifdef TI83PLUS
        ld      a,KG_1
        bcall(ab5_directIn)
        bit     DI_ALPHA,a
        pop     bc                              ; restore counter
        jr      z,longDelayAlpha                ; if [ALPHA] pressed, leave
#endif
#ifdef TI84PLUSCE
        call    keyScan
        ld      a,(kbdG2)
        bit     kbitAlpha,a
        pop     bc
        jr      nz,longDelayAlpha
#endif
        ei
        halt
        dec     bc
        ld      a,b
        or      c
        jr      nz,longDelayLoop
        or      a                               ; clear carry
longDelayDone:
        ret
longDelayAlpha:
        scf                                     ; set carry flag to denote [ALPHA] pressed
        jr      longDelayDone

#ifdef TI83PLUS
;------------------------------------------------
; spriteToRam - copy an 8x8 sprite to ram
;   input:  HL => sprite
;   output: DE => sprite in ram
;------------------------------------------------
spriteToRam:
        ld      de,sprBuffer
        push    de
        ld      bc,8
        ldir
        pop     de
        ret
#endif

#ifdef TI83PLUS
;------------------------------------------------
; swapBuffers - swap gbuf & videoBuffer
;   input:  none
;   output: none
;------------------------------------------------
swapBuffers:
        ld      hl,gbuf
        ld      ix,videoBuffer
        ld      bc,768
swapBuffersLoop:
        ; swap a byte
        ld      a,(hl)
        ld      d,(ix)
        ld      (hl),d
        ld      (ix),a
        ; inc pointers
        inc     hl
        inc     ix
        ; dec counter and check if zero, if not, loop
        dec     bc
        ld      a,c
        or      b
        jr      nz,swapBuffersLoop
        ret

;------------------------------------------------
; drawPixel - draw a pixel to gbuf
;   input:  A = x coord
;           HL => row of gbuf to use
;   output: none
;------------------------------------------------
drawPixel:
        ld      c,a
        srl     a
        srl     a
        srl     a
        ld de,0 \ ld e,a
        add     hl,de                           ; HL => byte of gbuf to modify
        ld      a,c
        and     %00000111
        or      a
        ld      b,a
        ld      a,%10000000
        jr      z,afterPixelShift
pixelShift:
        rrca
        djnz    pixelShift
afterPixelShift:
        or      (hl)
        ld      (hl),a
        ret
#endif

;------------------------------------------------
; divHLbyCs - divide HL by C (signed)
;   input:  HL = dividend
;           C = divisor
;   output: HL = quotient
;           A = remainder
;------------------------------------------------
divHLbyCs:
        xor     a
        bit     7,h
        jr      z,divHLbyCsGo
#ifdef TI84PLUSCE
        call    _setHLUto0
#endif
        ld a,h \ cpl \ ld h,a
        ld a,l \ cpl \ ld l,a
        inc     hl
        xor     a
        scf
divHLbyCsGo:
        push    af
#ifdef TI83PLUS
        ld      b,16
#endif
#ifdef TI84PLUSCE
        ld      b,24
#endif
divHLbyCsLoop:
        add     hl,hl
        rla
        cp      c
        jp      c,divHLbyCsNext
        sub     c
        inc     l
divHLbyCsNext:
        djnz    divHLbyCsLoop
        ld      b,a
        pop     af
        ld      a,b
        ret     nc
#ifdef TI84PLUSCE
        ld      a,$FF
        call    _setHLUtoA
#endif
        ld a,h \ cpl \ ld h,a
        ld a,l \ cpl \ ld l,a
        inc     hl
        ld      a,b
        ret

.end
