;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Miscellaneous routines (TI-84+CE only)                        ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; getDataPtr - take DE address from _chkfindsym and update it to point to start of data (whether in RAM or Archive)
;   input:  DE => _chkfindsym result
;   output: DE => start of file data
;   notes:  HL preserved
;------------------------------------------------
getDataPtr:
        push    hl    
        call    _SetAToDEU
        cp      $D0
        jr      nc,dataInRam
        push    de
        pop     ix
        ld      a,10
        add     a,(ix+9)
        or a \ sbc hl,hl \ ld l,a
        add     hl,de
        ex      de,hl
dataInRam:
        inc de \ inc de
        pop     hl
        ret

;------------------------------------------------
; checkDataVersion - check data file version number
;   input:  DE => version byte of data file
;   output: DE => start of data in data file
;------------------------------------------------
checkDataVersion:
        ld      a,(de)
        inc     de
        cp      DATA_FILE_VERSION
        ret     z
        pop     de                              ; clear RET from stack
        ; fall through to fileError

;------------------------------------------------
; fileError - show an error message for data file
;   input:  HL => string to show
;   output: none
;------------------------------------------------
fileError:
        push    hl
        call    _ClrScrn
        call    _HomeUp
        pop     hl
        call    _PutS
fileErrorLoop:
        call    _getcsc
        cp      GK_ENTER
        jr      nz,fileErrorLoop
        call    _ClrScrn
        call    _HomeUp
        ret

;------------------------------------------------
; delayBC - delay for a specified time (can be skipped by user)
;   input:  BC = delay length
;   output: CA = [ALPHA] was pressed, otherwise wasn't pressed
;------------------------------------------------
delayBC:
        dec     bc
        call    _setAtoBCU
        or      b
        or      c
        jr      nz,delayBC
        ret

.end
