;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Player routines                                               ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; playerUp - try to move player up
;   input:  none
;   output: none
;------------------------------------------------
playerUp:
        ld      hl,e3CheatFlags
        bit     reverseCheat,(hl)
        jp      nz,playerDownCheat
playerUpCheat:
        xor     a
        ld      (tempD),a                       ; set direction that player is actually trying to move (but not necessarily facing)
        call    checkFiring
        jr      nz,playerUpNormal               ; if retreat mode isn't enabled, walk normally
        ld      a,(ix+PD)
        dec     a                               ; is player facing down?
        jr      nz,playerUpNormal               ; if not, just walk normally
playerUpSlow:
        ld      hl,frame
        bit     0,(hl)
        ret     nz                              ; can only move every 2nd frame in retreat walking mode
        jr      playerUpCheck                   ; skip setting direction to "up"
playerUpNormal:
        xor     a
        ld      (ix+PD),a                       ; set direction
playerUpCheck:
        ; check if a solid tile is in the way
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
        push hl \ pop de
#endif
        ld      e,(ix+PX)
        ld      d,(ix+PX+1)
        inc     de
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        push    ix
        call    checkTile                       ; check tile at (px+1,py)
        pop     ix
        push    af                              ; save F flag
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
#endif
        ld      l,(ix+PX)
        ld      h,(ix+PX+1)
        ld      de,6
        add     hl,de
        ex      de,hl
#ifdef TI84PLUSCE
        ld      hl,0
#endif
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        push    ix
        call    checkTile                       ; check tile at (px+6,py)
        pop     ix
        pop     bc                              ; C = F flag from first checkTile
        bit     0,c
        jr      c,playerUpSlideL                ; if tile at top-right is solid, try to slide left
        jr      nz,playerUpSlideR               ; if tile at top-left is solid, try to slide right
        ; check if too close to top of screen
        ld      a,(ix+PSY)
;        cp      4
        cp      6
        ret     c                               ; if so, can't move
        dec     a                               ; update it
        ld      (ix+PSY),a
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        dec     hl
        ld      (ix+PY),l
        ld      (ix+PY+1),h                     ; save new y coord
        ret
playerUpSlideL:
        ret     nz                              ; if top-left was also solid, can't slide
        call    checkSmoothWalls
        ret     z                               ; if smooth walls option is disabled, can't slide
        jp      playerLeftSlow
playerUpSlideR:
        call    checkSmoothWalls
        ret     z
        jp      playerRightSlow

;------------------------------------------------
; playerDown - try to move player down
;   input:  none
;   output: none
;------------------------------------------------
playerDown:
        ld      hl,e3CheatFlags
        bit     reverseCheat,(hl)
        jp      nz,playerUpCheat
playerDownCheat:
        ld      a,1
        ld      (tempD),a
        call    checkFiring
        jr      nz,playerDownNormal
        ld      a,(ix+PD)
        or      a
        jr      nz,playerDownNormal
playerDownSlow:
        ld      hl,frame
        bit     0,(hl)
        ret     nz
        jr      playerDownCheck
playerDownNormal:
        ld      a,1
        ld      (ix+PD),a
playerDownCheck:
; check if a solid tile is in the way
#ifdef TI84PLUSCE
        ld      de,0
        ld      hl,0
#endif
        ld      e,(ix+PX)
        ld      d,(ix+PX+1)
        inc     de
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        ld      bc,7
        add     hl,bc
        push    ix
        call    checkTile                       ; check tile at (px+1,py+7)
        pop     ix
        push    af
#ifdef TI84PLUSCE
        ld      hl,0
#endif
        ld      l,(ix+PX)
        ld      h,(ix+PX+1)
        ld      de,6
        add     hl,de
        ex      de,hl
#ifdef TI84PLUSCE
        ld      hl,0
#endif
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        ld      bc,7
        add     hl,bc
        push    ix
        call    checkTile                       ; check tile at (px+6,py+7)
        pop     ix
        pop     bc
        bit     0,c
        jr      c,playerDownSlideL
        jr      nz,playerDownSlideR
        ; check if too close to bottom of screen
        ld      a,(ix+PSY)
#ifdef TI83PLUS
        cp      64-8-4
#endif
#ifdef TI84PLUSCE
        cp      (TILE_ROWS*8)-8-4
#endif
        ret     nc
        inc     a
        ld      (ix+PSY),a
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        inc     hl
        ld      (ix+PY),l
        ld      (ix+PY+1),h
        ret
playerDownSlideL:
        ret     nz
        call    checkSmoothWalls
        ret     z
        jp      playerLeftSlow
playerDownSlideR:
        call    checkSmoothWalls
        ret     z
        jp      playerRightSlow

;------------------------------------------------
; playerLeft - try to move player left
;   input:  none
;   output: none
;------------------------------------------------
playerLeft:
        ld      hl,e3CheatFlags
        bit     reverseCheat,(hl)
        jp      nz,playerRightCheat
playerLeftCheat:
        ld      a,2
        ld      (tempD),a
        call    checkFiring
        jr      nz,playerLeftNormal
        ld      a,(ix+PD)
        cp      3
        jr      nz,playerLeftNormal
playerLeftSlow:
        ld      hl,frame
        bit     0,(hl)
        ret     nz
        jr      playerLeftCheck
playerLeftNormal:
        ld      a,2
        ld      (ix+PD),a
playerLeftCheck:
; check if a solid tile is in the way
#ifdef TI84PLUSCE
        ld      de,0
        ld      hl,0
#endif
        ld      e,(ix+PX)
        ld      d,(ix+PX+1)
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        inc     hl
        push    ix
        call    checkTile                       ; check tile at (px,py+1)
        pop     ix
        push    af
#ifdef TI84PLUSCE
        ld      de,0
        ld      hl,0
#endif
        ld      e,(ix+PX)
        ld      d,(ix+PX+1)
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        ld      bc,6
        add     hl,bc
        push    ix
        call    checkTile                       ; check tile at (px,py+6)
        pop     ix
        pop     bc
        bit     0,c
        jr      c,playerLeftSlideU
        jr      nz,playerLeftSlideD
        ; check if too close to left of screen
        ld      a,(ix+PSX)
        cp      4
        ret     c
        dec     a
        ld      (ix+PSX),a
        ld      l,(ix+PX)
        ld      h,(ix+PX+1)
        dec     hl
        ld      (ix+PX),l
        ld      (ix+PX+1),h
        ret
playerLeftSlideU:
        ret     nz
        call    checkSmoothWalls
        ret     z
        jp      playerUpSlow
playerLeftSlideD:
        call    checkSmoothWalls
        ret     z
        jp      playerDownSlow

;------------------------------------------------
; playerRight - try to move player right
;   input:  none
;   output: none
;------------------------------------------------
playerRight:
        ld      hl,e3CheatFlags
        bit     reverseCheat,(hl)
        jp      nz,playerLeftCheat
playerRightCheat:
        ld      a,3
        ld      (tempD),a
        call    checkFiring
        jr      nz,playerRightNormal
        ld      a,(ix+PD)
        cp      2
        jr      nz,playerRightNormal
playerRightSlow:
        ld      hl,frame
        bit     0,(hl)
        ret     nz
        jr      playerRightCheck
playerRightNormal:
        ld      a,3
        ld      (ix+PD),a
playerRightCheck:
; check if a solid tile is in the way
#ifdef TI84PLUSCE
        ld      hl,0
#endif
        ld      l,(ix+PX)
        ld      h,(ix+PX+1)
        ld      de,7
        add     hl,de
        ex      de,hl
#ifdef TI84PLUSCE
        ld      hl,0
#endif
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        inc     hl
        push    ix
        call    checkTile                       ; check tile at (px+7,py+1)
        pop     ix
        push    af
#ifdef TI84PLUSCE
        ld      hl,0
#endif
        ld      l,(ix+PX)
        ld      h,(ix+PX+1)
        ld      de,7
        add     hl,de
        ex      de,hl
#ifdef TI84PLUSCE
        ld      hl,0
#endif
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        ld      bc,6
        add     hl,bc
        push    ix
        call    checkTile                       ; check tile at (p1x+7,p1y+6)
        pop     ix
        pop     bc
        bit     0,c
        jr      c,playerRightSlideU
        jr      nz,playerRightSlideD
        ; check if too close to right of screen
        ld      a,(ix+PSX)
#ifdef TI83PLUS
        cp      96-8-4
#endif
#ifdef TI84PLUSCE
        cp      (TILE_COLS*8)-8-4
#endif
        ret     nc
        inc     a
        ld      (ix+PSX),a
        ld      l,(ix+PX)
        ld      h,(ix+PX+1)
        inc     hl
        ld      (ix+PX),l
        ld      (ix+PX+1),h
        ret
playerRightSlideU:
        ret     nz
        call    checkSmoothWalls
        ret     z
        jp      playerUpSlow
playerRightSlideD:
        call    checkSmoothWalls
        ret     z
        jp      playerDownSlow

;------------------------------------------------
; playerUpB - move player up a set number of pixels
;   input:  B = # pixels to move
;   output: none
;------------------------------------------------
playerUpB:
        push    bc
        call    playerUp
        pop     bc
        djnz    playerUpB
        ret

;------------------------------------------------
; playerDownB - move player down a set number of pixels
;   input:  B = # pixels to move
;   output: none
;------------------------------------------------
playerDownB:
        push    bc
        call    playerDown
        pop     bc
        djnz    playerDownB
        ret

;------------------------------------------------
; playerLeftB - move player left a set number of pixels
;   input:  B = # pixels to move
;   output: none
;------------------------------------------------
playerLeftB:
        push    bc
        call    playerLeft
        pop     bc
        djnz    playerLeftB
        ret

;------------------------------------------------
; playerRightB - move player right a set number of pixels
;   input:  B = # pixels to move
;   output: none
;------------------------------------------------
playerRightB:
        push    bc
        call    playerRight
        pop     bc
        djnz    playerRightB
        ret

;------------------------------------------------
; teleportPlayer - teleport the player(s) based on the last teleport command
;   input:  none
;   output: none
;------------------------------------------------
teleportPlayer:
        ld      hl,(pTeleport)                  ; HL => coords to teleport to
        push    hl
        ld      l,(hl)
        call    setPlayerX
        pop     hl
        inc     hl
        ld      l,(hl)
        call    setPlayerY
#ifdef TI83PLUS
        bcall(ab5_fadeOut)
        bcall(_grbufclr)
        call    gbufCopy
        bcall(ab5_restoreContrast)
#endif
        jp      drawLevelJp

;------------------------------------------------
; playerUse - check for a usable tile in front of player and if one is found, use it
;   input:  none
;   output: none
;------------------------------------------------
playerUse:
        ld      ix,(pPlayer)
#ifdef TI83PLUS
        ld      h,0
        ld      l,(ix+PD)                       ; HL = player direction
        add     hl,hl
        add     hl,hl                           ; *4
#endif
#ifdef TI84PLUSCE
        ld      l,(ix+PD)
        ld      h,6
        mlt     hl
#endif
        ld      de,playerUseCoordsTable
        add     hl,de                           ; HL => required offsets
#ifdef TI83PLUS
        ld      c,(hl)
        inc     hl
        ld      b,(hl)                          ; BC = x offset
        inc     hl
#endif
#ifdef TI84PLUSCE
        call    _lddehl
        push de \ pop bc
#endif
        ex      de,hl                           ; DE => y offset
#ifdef TI84PLUSCE
        ld      hl,0
#endif
        ld      l,(ix+PX)
        ld      h,(ix+PX+1)
        add     hl,bc                           ; HL = x coord to check
        ex      de,hl                           ; DE = x coord to check; HL => y offset
#ifdef TI83PLUS
        ld      c,(hl)
        inc     hl
        ld      b,(hl)                          ; BC = y offset
#endif
#ifdef TI84PLUSCE
        push    de
        call    _lddehl
        push de \ pop bc
        pop     de
#endif
#ifdef TI84PLUSCE
        ld      hl,0
#endif
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        add     hl,bc                           ; HL = y coord to check
        call    getTile                         ; A = tile
        ; check table to see if it's a valid "use" tile
        ex      de,hl                           ; DE => tile in level data
        ld      bc,NUM_USE_TILES*256+0          ; B = num entries to check; C = index counter (starting at 0)
        ld      hl,playerUseTileTable           ; HL => start of table
checkPlayerUseTile:
        cp      (hl)                            ; check if tile matches an entry
        jr      z,playerUseFound                ; if it matches, go to next step
        inc     hl                              ; HL => next entry
        inc     c                               ; inc index counter
        djnz    checkPlayerUseTile              ; loop
        ret                                     ; no useable tiles found, so leave

playerUseFound:
#ifdef TI83PLUS
        ld h,0 \ ld l,c                         ; HL = index counter
        add     hl,hl                           ; *2
#endif
#ifdef TI84PLUSCE
        ld      h,3
        ld      l,c
        mlt     hl
#endif
        ld      bc,playerUseRoutines
        add     hl,bc                           ; HL => routine to execute
        LDHLIND()                               ; HL = routine to execute
        jp      (hl)                            ; go to routine; note that DE => tile in level data

;------------------------------------------------
; checkFiring - check if player is firing
;   input:  none
;   output: Z if firing, NZ if not
;------------------------------------------------
checkFiring:
        ld      hl,(pOptions)
        ld      a,(hl)
        and     OPTION_RETREAT                  ; is retreat firing enabled?
        jr      z,retreatDisabled
        ld      a,(coop)
        or      a
        jr      nz,checkFiring2P
        ; check for 1 player mode
#ifdef TI83PLUS
        ld      a,KG_TOP
        call    directIn
        bit     DI_2ND,a                        ; [2nd]?
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG1)
        xor     kbd2nd                          ; invert to get same result as 83+/84+ model
        bit     kbit2nd,a
#endif
        ret
checkFiring2P:
        ; determine which player we're checking
        ld      a,(player)
        or      a
        jr      nz,checkFiring2P2
checkFiring2P1:
#ifdef TI83PLUS
        ld      a,KG_TOP
        call    directIn
        bit     DI_WINDOW,a                     ; [WINDOW]?
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG1)
        xor     kbdWindow
        bit     kbitWindow,a
#endif
        ret
checkFiring2P2:
#ifdef TI83PLUS
        ld      a,KG_4
        call    directIn
        bit     DI_TAN,a
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG5)
        xor     kbdTan
        bit     kbitTan,a
#endif
        ret
retreatDisabled:
        inc     a                               ; set NZ
        ret

;------------------------------------------------
; checkSmoothWalls - check if smooth walls is enabled
;   input:  none
;   output: NZ if enabled, Z if not
;------------------------------------------------
checkSmoothWalls:
        ld      hl,(pOptions)
        ld      a,(hl)
        and     OPTION_WALLS
        ret

;------------------------------------------------
; drawPlayers - draw players to gbuf
;   input:  none
;   output: none
;------------------------------------------------
drawPlayers:
        ld      a,(p1p)
        xor     $01                             ; invert bit 0 of the active flag
        ld      b,a
        ld      a,(p1i)
        and     $01
        or      b
        ld      hl,p1scry
        ld      a,(p1d)
        ld      bc,sprPlayer1
#ifdef TI83PLUS
        ld      de,64
#endif
        ld      ix,p1ht
        call    z,drawPlayer                    ; only draw player 1 if active and bit 0 is reset of invincibility counter
        ld      a,(p2p)
        xor     $01
        ld      b,a
        ld      a,(p2i)
        and     $01
        or      b
        ld      hl,p2scry
        ld      a,(p2d)
        ld      bc,sprPlayer2
#ifdef TI83PLUS
        ld      de,-64
#endif
        ld      ix,p2ht
        ret     nz
        ; fall through to drawPlayer to draw player 2

;------------------------------------------------
; drawPlayer - draw a player to gbuf
;   input:  HL => x/y screen position (2 bytes)
;           A = player direction
;           BC => sprite set
;           DE = 64 or -64 (offset to other players sprite to use if this player is hurt, 83P only)
;           IX => player hurt flag
;   output: none
;------------------------------------------------
drawPlayer:
#ifdef TI83PLUS
        push    bc
        ld      c,(hl)                          ; C = player y screen position
        inc     hl
        ld      b,(hl)                          ; B = player x screen position
        pop     hl                              ; HL => sprite set
        push    bc                              ; save coords
        add     a,a
        add     a,a
        add     a,a
        add     a,a                             ; A = player direction * 16 (offset to sprite)
        ld bc,0 \ ld c,a
        add     hl,bc                           ; HL => masked sprite if player wasn't hurt this frame
        bit     0,(ix)                          ; was player hurt this frame?
        jr      z,doDrawPlayer                  ; if not, go and draw
        add     hl,de                           ; HL => masked sprite for hurt player (uses the other players sprite for this frame)
doDrawPlayer:
        pop     bc                              ; B,C = x,y coords respectively
        jp      putClippedMaskedSprite
#endif
#ifdef TI84PLUSCE
        push    bc
        ld      d,(hl)
        inc     hl
        ld      e,(hl)
        ld      l,a
        ld      h,128
        mlt     hl
        add     hl,hl
        pop     bc
        add     hl,bc                           ; HL => sprite
        bit     0,(ix)
        jp      z,drawTransSprite
        jp      drawTransSpriteWhite
#endif

;------------------------------------------------
; checkTile - check a tile to see if it's a wall, trigger, pickup item, etc.
;   input:  DE = x
;           HL = y
;   output: CA = 1 if can't walk through
;------------------------------------------------
checkTile:
        call    trgWalkedOnArea                 ; trigger event
        call    getTile                         ; A = tile number
        ; do the checks for the intex tip
        push    af
        push    hl
        ld      a,(episode)
        or      a
        jr      nz,afterIntexTip                ; if not episode 1, no check for the intex tip
        ld      a,(level)
        or      a
        jr      nz,afterIntexTip                ; if not level 1, no check for the intex tip
        ld      de,(tileDataPtr)
        sbc     hl,de                           ; HL = offset to tile in tile data
        ld      de,TIP_INTEX_OFFSET
        CPHLDE()                                ; check if it's the intex tip trigger tile
        jr      nz,afterIntexTip
        ld      a,TIP_INTEX
        ld      d,0
        bcall(ab5_showTip)                      ; show tip for intex terminals
afterIntexTip:
        pop     hl
        pop     af
        cp      TILE_DOOR                       ; is it a key door?
        jr      nc,tryOpenDoor                  ; if so, try to open it
        cp      TILE_UNOPENALIENHOLE            ; is it an unopen alien hole?
        jr      z,activateAlienHole             ; if so, try to activate it
        cp      TILE_BOMB                       ; is it a bomb/mine?
        jr      z,triggerBombMine               ; if so, trigger bomb/mine
        cp      TILE_ENEMYTRIGGER+1             ; is it an enemy2 trigger?
        jp      z,checkTileMove                 ; if so, can always walk on it (in case player is on bare/exposed floor)
        cp      TILE_FIREDOORV
        jp      z,checkFireDoorTip
        cp      TILE_FIREDOORH
        jp      z,checkFireDoorTip
        ; if player is standing on tile 0 (bare/exposed floor type), then value is modified so that the player
        ; can only walk on tiles 0-4 inclusive; this check is performed in the main game loop
        ld      ix,(pPlayer)
        cp      (ix+PCT)                        ; check tile against player wall collision tile
        jr      c,checkEnergyShield             ; if not, check if it's an energy shield
checkTileNoMove:
        call    trgHitWall
        ld      hl,e1CheatFlags
        or      a                               ; clear carry
        bit     noWallsCheat,(hl)               ; cheat active?
        ret     nz                              ; if so, leave with clear carry (can walk through wall)
        scf                                     ; otherwise set carry (can't walk through wall)
#ifdef NO_CLIP
        or      a
#endif
        ret
activateAlienHole:
        call    loadAlienHole                   ; try to load alien hole if one isn't already active
        jr      checkTileMove                   ; ok to move
triggerBombMine:
        ld      a,(levelType)
        and     $0F
        jr      z,triggerBomb
triggerMine:
        ld      a,TILE_CRATER
        call    putTile
        call    getExactXYFromHL
        ld      bc,0*256+ANIM_EXPLOSION         ; B = animation type; C = counter ini
        call    newAnim
        ld      hl,(pPlayer)
        ld      bc,PH
        add     hl,bc
#ifndef INVINCIBLE
        ld      a,(hl)
        sub     MINE_DAMAGE
        ld      (hl),a                          ; save it
        ld      bc,PHT-PH
        add     hl,bc                           ; HL => player hurt flag
        ld      (hl),1                          ; set flag for player getting hurt this frame
#endif
        jr      checkTileMove
triggerBomb:
        ld      a,TILE_TRIGGEREDBOMB            ; A = tile to set
        call    putTile                         ; place the tile & display it
        call    trgLifeBomb
        jr      checkTileMove                   ; ok to move
tryOpenDoor:
        ld      ix,e3CheatFlags
        bit     keysCheat,(ix)
        jr      nz,tryOpenDoorOK
        ld      ix,(pPlayer)
        ld      a,(ix+PK)
        or      a                               ; does player have any key passes?
        jp      z,doorNoKeys                    ; if not, can't move
        dec     (ix+PK)                         ; use a key
tryOpenDoorOK:
        ld      a,(hl)                          ; A = tile number
        call    openDoor                        ; open the door
        jr      checkTileMove                   ; indicate that it's ok to move and ret
checkEnergyShield:
        cp      TILE_ENERGYSHIELD               ; is tile an energy shield
        jr      c,checkPickups                  ; if not, check pickups
        sub     TILE_ENERGYSHIELD               ; A = direction of energy shield
        ld      hl,tempD
        cp      (hl)                            ; compare energy shield direction to player movement direction
        jr      nz,checkTileEnergyDeath         ; can't move if directions don't match, player might die if facing into the one-way beam
checkTileMove:
        or      a                               ; clear carry
        ret
checkTileEnergyDeath:
        xor     %00000001                       ; A = reverse direction of energy beam
        cp      (hl)                            ; check it to energy shield direction
        jp      nz,checkTileNoMove              ; if they don't match, don't kill the player
        ld      hl,(pPlayer)
        ld      de,PH
        add     hl,de
        ld      (hl),-1                         ; player is dead
        jp      checkTileNoMove
checkPickups:
        push    hl                              ; save tile data ptr
        sub     PICKUPS                         ; is tile a key pass?
        jr      z,getKeyPass
        dec     a                               ; is it ammo?
        jr      z,getAmmo
        dec     a                               ; is it 10 credits?
        jp      z,get10Credits
        dec     a                               ; is it 100 credits?
        jp      z,get100Credits
        dec     a                               ; is it first aid?
        jp      z,getFirstAid
        dec     a                               ; is it an extra life?
        jp      z,getExtraLife
        pop     hl                              ; restore tile data ptr
        or      a                               ; no pickup, so clear carry and leave
        ret
getKeyPass:
        ld      ix,(pPlayer)
        inc     (ix+PK)                         ; add a key
        ; increment key pass achievement
        ld      bc,ACH_KEY1_BIT*256+ACH_KEY1_ID
        ld      de,ACH_KEY1_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      bc,ACH_KEY2_BIT*256+ACH_KEY2_ID
        ld      de,ACH_KEY2_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      a,TIP_KEYS
        ld      d,0
        bcall(ab5_showTip)                      ; show tip for key passes
        jp      getPickupScore
getAmmo:
        ; increment ammo clip achievement
        ld      bc,ACH_AMMO1_BIT*256+ACH_AMMO1_ID
        ld      de,ACH_AMMO1_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      bc,ACH_AMMO2_BIT*256+ACH_AMMO2_ID
        ld      de,ACH_AMMO2_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      hl,getPickupScore               ; HL => where to ret to
        push    hl                              ; put it on the stack
addAmmoClip:
        ld      ix,(pPlayer)
        ld      de,PA
        add     ix,de
        ld      l,(ix)
        ld      h,(ix+1)
        ld      l,255                           ; L = full clip
        inc     h                               ; H = number of spare clips
        ld      c,4+1
        ld      a,(difficulty)
        or      a
        jr      nz,addAmmoClipCheck
        ld      c,8+1
addAmmoClipCheck:
        ld      a,h
        cp      c                               ; check to make sure no more than C-1 spare clips
        jr      c,setAmmo
        ld      h,c
        dec     h
setAmmo:
        ld      (ix),l
        ld      (ix+1),h
        ret                                     ; ret either to getPickupScore or intexBuyAmmoClip depending on where this routine was called from
get10Credits:
        ld      de,10
        jr      getCredits
get100Credits:
        ld      de,100
getCredits:
        ld      hl,(credits)
        add     hl,de                           ; add credits
        ld      (credits),hl
        ex      de,hl                           ; HL = # credits picked up
        ; increment credits achievements
        push    hl
        ld      bc,ACH_CREDIT1_BIT*256+ACH_CREDIT1_ID
        ld      de,ACH_CREDIT1_OFFSET
        bcall(ab5_incAchievement)
        pop     hl
        push    hl
        ld      bc,ACH_CREDIT2_BIT*256+ACH_CREDIT2_ID
        ld      de,ACH_CREDIT2_OFFSET
        bcall(ab5_incAchievement)
        pop     hl
        push    hl
        ld      bc,ACH_CREDIT3_BIT*256+ACH_CREDIT3_ID
        ld      de,ACH_CREDIT3_OFFSET
        bcall(ab5_incAchievement)
        pop     hl
        ld      bc,ACH_CREDIT4_BIT*256+ACH_CREDIT4_ID
        ld      de,ACH_CREDIT4_OFFSET
        bcall(ab5_incAchievement)
        ld      a,TIP_CREDITS
        ld      d,0
        bcall(ab5_showTip)                      ; show tip for credits
        jr      getPickupScore
getFirstAid:
        ; increment achievement
        ld      bc,ACH_FIRSTAID1_BIT*256+ACH_FIRSTAID1_ID
        ld      de,ACH_FIRSTAID1_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      hl,(pPlayer)
        ld      de,PSH
        add     hl,de
        ld      a,90
        ld      (hl),a                          ; set counter to show health meter for the next A frames
        ld      de,PH-PSH
        add     hl,de
        ld      a,MAX_HEALTH/2
        add     a,(hl)                          ; add health
        ld      (hl),a
        cp      MAX_HEALTH                      ; check it's not over the max
        jr      c,getPickupScore
        ld      (hl),MAX_HEALTH                 ; if it was, set it to MAX_HEALTH
        jr      getPickupScore
getExtraLife:
        ld      bc,ACH_EXTRALIVES1_BIT*256+ACH_EXTRALIVES1_ID
        ld      de,ACH_EXTRALIVES1_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      hl,(pPlayer)
        ld      de,PL
        add     hl,de
        inc     (hl)                            ; add a life
getPickupScore:
        ld      hl,SCORE_PICKUP
        ld      ix,(pPlayer)
        call    addScore                        ; increase score
        ld      hl,numPickups
        inc     (hl)                            ; inc pickups counter for this level
        pop     hl                              ; restore tile data ptr
        ld      a,TILE_FLOOR
        call    putTile                         ; clear pickup from level
        or      a                               ; clear carry flag
        ret
doorNoKeys:
        ld      a,TIP_DOORS
        ld      d,0
        bcall(ab5_showTip)                      ; show tip for opening doors
        jp      checkTileNoMove
checkFireDoorTip:
        ld      a,TIP_FIREDOORS
        ld      d,0
        bcall(ab5_showTip)                      ; show tip for fire doors
        jp      checkTileMove

;------------------------------------------------
; addScoreDiff - add to the players score, adjusting for difficulty
;   input:  HL = base value to add before adjusting for difficulty
;           IX => player data
;   output: HL = new score
;------------------------------------------------
addScoreDiff:
        ld      a,(difficulty)
        or      a                               ; check difficulty
        jr      z,addScore                      ; if NORMAL, don't x1.5 the value
        ld      d,h
        ld      e,l
        srl     h
        rr      l
        add     hl,de
        ; fall through to addScore

;------------------------------------------------
; addScore - add to the players score
;   input:  HL = value to add
;           IX => player data
;   output: HL = new score
;------------------------------------------------
addScore:
        ; can only add score if player is active
        ld      a,(ix+PP)
        or      a
        ret     z
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(ix+PS)
        ld      d,(ix+PS+1)
        add     hl,de
        ld      (ix+PS),l
        ld      (ix+PS+1),h                     ; save new score
        ret

;------------------------------------------------
; selectWeaponX - select a weapon
;   input:  none
;   output: none
;------------------------------------------------
selectWeapon1:
        xor     a
        inc     a                               ; A = weapon #, and also NZ (player always has this weapon)
selectWeapon:
        ret     z                               ; leave it Z (player doesn't have this weapon)
        ld      hl,(pPlayer)
        ld      de,PW
        add     hl,de
        ld      (hl),a                          ; set new weapon type
        ret
selectWeapon2:
        ld      a,2                             ; A = weapon #
        ld      hl,weaponTable
        bit     1,(hl)                          ; check if weapon available
        jr      selectWeapon
selectWeapon3:
        ld      a,3                             ; A = weapon #
        ld      hl,weaponTable
        bit     2,(hl)                          ; check if weapon available
        jr      selectWeapon
selectWeapon4:
        ld      a,4                             ; A = weapon #
        ld      hl,weaponTable
        bit     3,(hl)                          ; check if weapon available
        jr      selectWeapon
selectWeapon5:
        ld      a,5                             ; A = weapon #
        ld      hl,weaponTable
        bit     4,(hl)                          ; check if weapon available
        jr      selectWeapon

;------------------------------------------------
; checkShowHealth - check if should be showing health meter for this player
;   input:  HL => where on gbuf to start bottom of meter
;           BC => show health counter
;           DE => health
;   output: none
;------------------------------------------------
checkShowHealth:
        ld      a,(bc)
        or      a                               ; check if should be showing health meter
        ret     z                               ; if not, skip it
        dec     a
        ld      (bc),a                        ; dec counter and save it
        ld      a,(de)
        ; fall through to displayMeter

;------------------------------------------------
; displayMeter - display a meter
;   input:  A = value to be used (either health, or ammo/2)
;           HL => where on gbuf to start bottom of meter
;   output: none
;------------------------------------------------
displayMeter:
        srl     a
        srl     a                               ; A = (0-31)
        ld      c,a                             ; C = meter fill value
        ld      de,-12                          ; DE = offset to next line up on gbuf
        ld      a,(hl)
        and     %10000000
        ld      (hl),a                          ; clear white space below meter
        add     hl,de                           ; HL => next line up on gbuf
        ld      a,(hl)
        and     %10000000
        or      %00111110
        ld      (hl),a                          ; draw bottom border of meter
        add     hl,de                           ; HL => next line up on gbuf
        ld      b,31                            ; B = 31 lines of meter to draw
displayMeterLoop:
        push    de                              ; save gbuf line offset
        ld      a,(hl)                          ; get byte from gbuf
        and     %10000000                       ; apply mask to clear where meter is displayed
        or      %00100010                       ; apply meter side outlines
        ld      d,a                             ; D = potential byte to be written back to gbuf
        ld      a,c                             ; A = meter value counter
        or      a                               ; check if it's 0
        ld      a,d                             ; get byte back
        jr      z,displayMeterDraw              ; if so, just draw the byte already have
        dec     c                               ; dec meter value counter
        or      %00011100                       ; apply meter fill to byte for gbuf
displayMeterDraw:
        ld      (hl),a                          ; write byte back to gbuf
        pop     de                              ; restore gbuf line offset
        add     hl,de                           ; HL => next line up on gbuf
        djnz    displayMeterLoop
        ld      a,(hl)
        and     %10000000
        or      %00111110
        ld      (hl),a                          ; draw top border of meter
        add     hl,de                           ; HL => next line up on gbuf
        ld      a,(hl)
        and     %10000000
        ld      (hl),a                          ; clear white space above meter
        ret

;------------------------------------------------
; playerIntex - start up the intex routines (located on app page 0)
;   input:  none
;   output: none
;------------------------------------------------
playerIntex:
        bcall(ab5_loadIntexConsole)             ; run the intex routines on page 0
        ret

;------------------------------------------------
; checkPlayerDead - check if player is dead and if so, handle the death spin, etc.
;   input:  IX => player data
;           HL => player dead variable (p1dd or p2dd)
;           BC => player direction
;           DE => player lives
;           IX => player health
;   output: none
;------------------------------------------------
checkPlayerDead:
        ld      a,(ix+PP)
        or      a
        ret     z                               ; nothing to do if player isn't active
        ld      a,(ix+PDD)
        or      a
        ret     z                               ; if player is alive, nothing to do
        dec     (ix+PDD)                        ; dec death spin counter
        jr      nz,afterEndDeathSpin            ; if counter hasn't reached zero, don't try to bring player back to life yet
        ld      a,(ix+PL)
        bit     7,a                             ; check if lives has gone under zero
        jr      nz,playerDeadNoLives
        ld      (ix+PI),INI_INVINCIBLE          ; when player first comes back to life, they are temporarily invincible
        ld      (ix+PH),MAX_HEALTH              ; fill up player health
afterEndDeathSpin:
        ld      a,(ix+PDD)
        and     %00000011
        ret     nz                              ; only update death spin every 4th frame
        ld      hl,0
        ld      l,(ix+PD)                       ; HL = player direction
        ld      de,deathSpinTable
        add     hl,de
        ld      a,(hl)                          ; A = new direction
        ld      (ix+PD),a                       ; save it
        ret
playerDeadNoLives:
        ld      (ix+PP),0                       ; player no longer active
        ret

;------------------------------------------------
; checkPlayerDied - check if player died this frame
;   input:  HL => player dead flag
;           DE => player health
;           BC => player lives
;   output: none
;------------------------------------------------
checkPlayerDied:
        ld      a,(de)
        bit     7,a                             ; if bit 7 is set, player health is under zero
        ret     z
        ld      (hl),INI_DEAD                   ; set player as dead with death spin counter
        xor     a
        ld      (de),a                          ; set health to zero so that health meter displays as empty
        ld      a,(bc)
        dec     a                               ; take a player life
        ld      (bc),a
        ret

;------------------------------------------------
; checkPlayerInvincible - check if player is invincible, and if so decrement the counter
;   input:  HL => player invincible counter
;   output: none
;------------------------------------------------
checkPlayerInvincible:
        ld      a,(hl)
        or      a                               ; is player invincible?
        ret     z                               ; if not, no need to dec invincible counter
        dec     (hl)                            ; dec playerInv
        ret

;------------------------------------------------
; calcPlayerCt - check what tile should be used for wall collisions for player this frame
;   input:  IX => player data
;   output: none
;------------------------------------------------
calcPlayerCt:
#ifdef TI84PLUSCE
        ld      de,0
        ld      hl,0
#endif
        ld      e,(ix+PX)
        ld      d,(ix+PX+1)
        inc     de                              ; DE = player x+1
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        inc     hl                              ; HL = player y+1
        call    getTile                         ; A = tile at top left of player
        call    checkE2Trigger                  ; if the tile is an enemy 2 trigger, check the tile to the right of the player instead
        or      a                               ; is it bare/explosed floor?
        jr      nz,setNormalWallCheck           ; if not, player is considered to be on normal floor
#ifdef TI84PLUSCE
        LDHL0()
#endif
        ld      l,(ix+PX)
        ld      h,(ix+PX+1)
        ld      de,6
        add     hl,de
        ex      de,hl                           ; DE = player x+6
#ifdef TI84PLUSCE
        LDHL0()
#endif
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        ld      bc,6
        add     hl,bc                           ; HL = player y+6
        call    getTile                         ; A = tile at bottom right of player
        call    checkE2Trigger                  ; if the tile is an enemy 2 trigger, check the tile to the right of the player instead
        or      a                               ; is it bare/exposed floor?
        jr      nz,setNormalWallCheck           ; if not, player is considered to be on normal floor
        ld      a,TILE_FLOOR                    ; any tile that's not bare floor or a ramp will be considered a wall
        jr      setWallCheck
setNormalWallCheck:
        ld      a,TILE_SOLID
setWallCheck:
        ld      (ix+PCT),a                      ; tile number to use for player/wall collision
        ret

;------------------------------------------------
; checkCentreTile - check tile at centre of player
;   input:  IX => player data
;   output: none
;------------------------------------------------
checkCentreTile:
        ld      (pPlayer),ix
        ld      e,(ix+PX)
        ld      d,(ix+PX+1)
        inc     de
        inc     de
        inc     de                              ; DE = centre of player x
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        inc     hl
        inc     hl
        inc     hl                              ; HL = centre of player y
        call    getTile                         ; A = tile
        cp      TILE_GARBAGECHUTE               ; is it a garbage chute?
        jr      nz,afterGarbageChute
        ld      a,(frame)
        cp      128
        ret     c                               ; ok to walk on
        cp      255-8
        ret     nc                              ; ok to walk on
        
checkTileDeath:
        ; check if player is dead or invincible
        ld      a,(ix+PI)
        or      (ix+PDD)
        ret     nz
        ; check for invincibility cheat
        ld      a,(e1CheatFlags)
        bit     invincibleCheat,a
        ret     nz
        ld      a,-1
        ld      (ix+PH),a                       ; set player as dead
        ret

afterGarbageChute:
        cp      TILE_VENTFAN                    ; is it a vent fan?
        jr      nz,afterVentFan
        jr      checkTileDeath

afterVentFan:
        cp      TILE_COMPACTORL                 ; is it a compactor?
        jr      z,checkCompactor
        cp      TILE_COMPACTORR
        jr      nz,afterCompactor
checkCompactor:
        ld      a,(frame)
        and     %00011100
        cp      %00010000
        ret     nz                              ; ok to walk on
        jr      checkTileDeath

afterCompactor:
        cp      TILE_AIRDUCT                    ; is it an air duct?
        jr      c,afterAirDuct                  ; if not, skip this
        cp      TILE_AIRDUCT+4
        jr      nc,afterAirDuct
        ; check top-left of player to see if on TILE_AIRDUCT
        ; if so, check bottom-right of player to see if on TILE_AIRDUCT+3
#ifdef TI83PLUS
        ld      e,(ix+PX)
        ld      d,(ix+PX+1)
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        call    getTile
        cp      TILE_AIRDUCT
        jr      nz,afterAirDuct
        ld      l,(ix+PX)
        ld      h,(ix+PX+1)
        ld      bc,7
        add     hl,bc
        ex      de,hl
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        add     hl,bc
        call    getTile
        cp      TILE_AIRDUCT+3
        jr      nz,afterAirDuct
#endif
#ifdef TI84PLUSCE
        ld      de,(ix+PX)
        ld      hl,(ix+PY)
        call    getTile
        cp      TILE_AIRDUCT
        jr      nz,afterAirDuct
        ld      hl,(ix+PX)
        ld      bc,7
        add     hl,bc
        ex      de,hl
        ld      hl,(ix+PY)
        add     hl,bc
        call    getTile
        cp      TILE_AIRDUCT+3
        jr      nz,afterAirDuct
#endif
        jr      checkTileDeath

afterAirDuct:
        cp      TILE_PUSHUP
        jr      z,doPushup
        cp      TILE_PUSHDOWN
        jr      nz,afterPushTile
        ld      hl,playerDownCheck              ; skips the direction setting part in playerDown
        jr      doPush
doPushup:
        ld      hl,playerUpCheck
doPush:
        ld      a,(frame)
        bit     0,a
        ret     nz                              ; can only push every 2nd frame
        jp      (hl)                            ; jp to either playerUpCheck or playerDownCheck then ret

afterPushTile:
        cp      TILE_ENERGYSHIELD+4             ; check upper bound if tile is an energy shield
        ret     nc                              ; if not, no pushing
        sub     TILE_ENERGYSHIELD               ; check lower bound if tile is an energy shield
        ret     c                               ; if not, no pushing to do
        ld      b,2
        or      a                               ; energy shield up?
        jp      z,playerUpB                     ; if so, move player up 2 pixels
        dec     a                               ; energy shield down?
        jp      z,playerDownB                   ; if so, move player down 2 pixels
        dec     a                               ; left?
        jp      z,playerLeftB
        jp      playerRightB                    ; must be right

.end
