;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Screen scrolling routines (TI-84+CE version)                  ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; scrollUp - scroll the screen up a pixel
;   input:  none
;   output: none
;------------------------------------------------
scrollUp:
        ; first, check that we're not already viewing the bottom of the level
        ld      hl,(screenY)
        ld      de,TILE_ROWS*8
        add     hl,de
        ex      de,hl                           ; DE = y pos of bottom of screen + 1
        ld      hl,(levelHeight)
        ld      h,8
        mlt     hl                              ; HL = level height in pixels
        call    _cphlde                         ; compare against bottom of screen
        ret     z                               ; if they equal, we're at the bottom of the level, so don't scroll
        ld      hl,p1scry
        dec     (hl)                            ; player will move up 1 pixel on screen due to the scroll
        ld      hl,p2scry
        dec     (hl)
        ld      hl,(screenY)
        push    hl
        ld      a,l
        and     %00000111
        jr      nz,suAfterTriggerCheck
        ld      de,TILE_ROWS*8
        add     hl,de
        ld      de,(screenX)
        call    getTile                         ; HL => tile just off bottom of screen (at the left)
        ld      b,TILE_COLS+1
suTriggerLoop:
        push    bc
        call    checkTrigger
        inc     hl
        pop     bc
        djnz    suTriggerLoop
suAfterTriggerCheck:
        pop     hl
        inc     hl
        ld      (screenY),hl
        ret

;------------------------------------------------
; scrollDown - scroll the screen down a pixel
;   input:  none
;   output: none
;------------------------------------------------
scrollDown:
        ; first, check that we're not already viewing the top of the level
        ld      hl,(screenY)
        call    _chkHLis0                       ; is screen y pos 0?
        ret     z                               ; if so, we're at the top of the level, so don't scroll
        ld      hl,p1scry
        inc     (hl)                            ; player will move down 1 pixel on screen as a result of the scroll
        ld      hl,p2scry
        inc     (hl)
        ld      hl,(screenY)
        push    hl
        ld      a,l
        and     %00000111
        jr      nz,sdAfterTriggerCheck
        dec     hl
        ld      de,(screenX)
        call    getTile                         ; HL => tile just off bottom of screen (at the left)
        ld      b,TILE_COLS+1
sdTriggerLoop:
        push    bc
        call    checkTrigger
        inc     hl
        pop     bc
        djnz    sdTriggerLoop
sdAfterTriggerCheck:
        pop     hl
        dec     hl
        ld      (screenY),hl
        ret

;------------------------------------------------
; scrollLeft - scroll the screen left a pixel
;   input:  none
;   output: none
;------------------------------------------------
scrollLeft:
        ; first, check that we're not already viewing the far right of the level
        ld      hl,(screenX)
        ld      de,TILE_COLS*8
        add     hl,de
        ex      de,hl                           ; DE = x pos of right of screen + 1
        ld      hl,(levelWidth)
        ld      h,8
        mlt     hl                              ; HL = level width in pixels
        call    _cphlde                         ; compare against right of screen
        ret     z                               ; if they equal, we're at the far right of the level, so don't scroll
        ld      hl,p1scrx
        dec     (hl)                            ; player will move left 1 pixel on screen due to the scroll
        ld      hl,p2scrx
        dec     (hl)
        ld      hl,(screenX)
        push    hl
        ld      a,l
        and     %00000111
        jr      nz,slAfterTriggerCheck
        ld      de,TILE_COLS*8
        add     hl,de
        ex      de,hl
        ld      hl,(screenY)
        call    getTile                         ; HL => tile just off bottom of screen (at the left)
        ld      b,TILE_ROWS+1
        ld      a,(screenY)
        and     %00000111
        jr      nz,slScan
        dec     b
slScan:
        ld      a,(levelWidth)
        ld de,0 \ ld e,a
slTriggerLoop:
        push    bc
        push    de
        call    checkTrigger
        pop     de
        add     hl,de
        pop     bc
        djnz    slTriggerLoop
slAfterTriggerCheck:
        pop     hl
        inc     hl
        ld      (screenX),hl
        ret

;------------------------------------------------
; scrollRight - scroll the screen right a pixel
;   input:  none
;   output: none
;------------------------------------------------
scrollRight:
        ; first, check that we're not already viewing the far left of the level
        ld      hl,(screenX)
        call    _chkHLis0                       ; is screen x pos 0?
        ret     z                               ; if so, we're at the far left of the level, so don't scroll
        ld      hl,p1scrx
        inc     (hl)                            ; player will move right 1 pixel on screen as a result of the scroll
        ld      hl,p2scrx
        inc     (hl)
        ld      hl,(screenX)
        push    hl
        ld      a,l
        and     %00000111
        jr      nz,srAfterTriggerCheck
        dec     hl
        ex      de,hl
        ld      hl,(screenY)
        call    getTile                         ; HL => tile just off bottom of screen (at the left)
        ld      b,TILE_ROWS+1
        ld      a,(screenY)
        and     %00000111
        jr      nz,srScan
        dec     b
srScan:
        ld      a,(levelWidth)
        ld de,0 \ ld e,a
srTriggerLoop:
        push    bc
        push    de
        call    checkTrigger
        pop     de
        add     hl,de
        pop     bc
        djnz    srTriggerLoop
srAfterTriggerCheck:
        pop     hl
        dec     hl
        ld      (screenX),hl
        ret

;------------------------------------------------
; checkTrigger - check for enemy & object triggers
;   input:  HL => tile in level data to check
;   output: HL => tile in level data
;------------------------------------------------
checkTrigger:
        push    hl
        call    checkEnemyTrigger
        pop     hl
        push    hl
        call    checkObjectTrigger
        pop     hl
        ret

.end
