;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Text routines (TI-84+CE only)                                 ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; printBlockHL - same as printBlock, but starts with HL as the pointer
;   input:  HL => string block
;   output: IX => data after block
;------------------------------------------------
printBlockHL:
        push    hl
        pop     ix
        ; fall through to printBlock

;------------------------------------------------
; printBlock - print a block of strings to vbuf
;   input:  IX => string block
;   output: IX => data after block
;------------------------------------------------
printBlock:
        ld      b,(ix)                          ; B = # strings
        inc     ix                              ; IX => first string data (row, col/2, string)
pbLoop:
        push    bc
        ld      l,(ix)
        inc     ix
        ld      e,(ix)
        ld      d,2
        mlt     de
        inc     ix                              ; IX => actual string
        call    printString
        pop     bc
        djnz    pbLoop
        ret

;------------------------------------------------
; setShowHL/showHL - show HL in small font at specified position (right aligned)
;   input:  HL = value to show
;           DE = x pos
;           C = y pos
;   output: DE = x pos after end of string
;           L = y pos
;------------------------------------------------
setShowHL:
showHL:
        push    de
        push    bc
        ld      de,showHLBuffer+8
        ld      b,8
        xor     a
        ld      (de),a
showHLMake:
        dec     de
        ld      a,10
        call    _divHLbyA
        add     a,'0'
        ld      (de),a
        djnz    showHLMake
        ex      de,hl                           ; HL => start of string
        ; replace leading zeros with spaces
        push    hl
        ld      b,7
showHLCheck:
        ld      a,(hl)
        cp      '0'
        jr      nz,showHLReady
        ld      (hl),' '
        inc     hl
        djnz    showHLCheck
showHLReady:
        pop     ix                              ; IX => string
        pop     hl
        pop     de
        jr      printString

;------------------------------------------------
; showHLLeft - show HL in small font at specified position (left aligned)
;   input:  HL = value to show
;           DE = x pos
;           C = y pos
;   output: DE = x pos after end of string
;           L = y pos
;------------------------------------------------
showHLLeft:
        push    bc
        push    de
        ld      de,showHlBuffer+8
        ld      b,8
        xor     a
        ld      (de),a
showHLLeftMake:
        dec     de
        ld      a,10
        call    _divHLByA
        add     a,'0'
        ld      (de),a
        djnz    showHLLeftMake
        push    de
        pop     ix                              ; IX => start of string
        ld      b,7
showHLLeftCheck:
        ld      a,(ix)
        cp      '0'
        jr      nz,showHLLeftReady
        inc     ix
        djnz    showHLLeftCheck
showHLLeftReady:
        pop     de
        pop     hl
        jr      printString

;------------------------------------------------
; printStringC - set the colour and print a string to vbuf
;   input:  A = colour
;           IX => string
;           DE = x pos
;           L = y pos
;   output: IX => data after string
;           DE = x pos after end of string
;           L = y pos
;------------------------------------------------
printStringC:
        ld      (__psColour),a
        ; fall through to printString

;------------------------------------------------
; printString - print a string to vbuf
;   input:  IX => string
;           DE = x pos
;           L = y pos
;   output: IX => data after string
;           DE = x pos after end of string
;           L = y pos
;------------------------------------------------
printString:
        ld      a,(ix)
        inc     ix
        or      a                               ; NULL terminator?
        ret     z
        push    ix
        push    hl
        push    de
        ; convert any lowercase to uppercase
        cp      96
        jr      c,psLower
        sub     32
psLower:
        sub     $20
        ld      c,a
        ld      b,11
        mlt     bc
        ld      ix,chars
        add     ix,bc                           ; IX => char sprite
        ld      h,160
        mlt     hl
        add     hl,hl
        add     hl,de
        ld      de,(pVbuf)
        add     hl,de                           ; HL => where on vbuf to draw character
        ld      bc,11*256+128                   ; B = 11 rows, C = %10000000 bitmask
        ld      de,320                          ; DE = 320 bytes of vbuf to skip per row (used at end of each row)
psRow:
        push    hl
psCol:
        ld a,(ix) \ and c                       ; check bit
        jr      z,psNext
__psColour              = $+1
        ld      (hl),$FF
psNext:
        inc     hl
        rrc     c
        jr      nc,psCol                        ; if bitmask hasn't fully rotated, do next column
        inc     ix                              ; IX => next byte of sprite
        pop     hl
        add     hl,de                           ; HL => where on vbuf to start drawing next row
        djnz    psRow
        pop     de
        ld      hl,8
        add     hl,de
        ex      de,hl                           ; DE = x pos for next character
        pop     hl                              ; L = y pos
        pop     ix                              ; IX => next character
        jr      printString

;------------------------------------------------
; loadChars - extract character sprites
;   input:  none
;   output: none
;------------------------------------------------
loadChars:
        ld      hl,(pDatFile)
        ld      de,fontce
        add     hl,de
        ld      de,chars
        ld      ix,huffTree
        jp      HuffExtr

.end
